""" Etude de la charge  du circuit RC piloté avec Arduino
Attention: le moniteur série d'Arduino doit toujours être fermé avant d'activer le port série dans Python
La carte Arduino ne peut pas communiquer simultanément avec le moniteur série Arduino et avec Python
Le port série doit être fermé par la commande port_serie.close() dans Python pour ouvrir le moniteur Arduino
// Téleverser le programme 'DechargeRCauto-eleve.ino'dans Arduino puis lancer le programme Python //
"""
import serial # gestion du port série. installer la librairie pyserial si besoin
import time # module de gestion du temps 
import numpy as np 
import matplotlib . pyplot as plt # pour faire des graphiques
import os # os: operating system: module permettant de dialoguer avec le système d'exploitation
from matplotlib.widgets import Cursor

# entrée des paramètres du circuit RC et des fichiers de travail
R = float(input("Entrer la valeur de R (en ohm) :   "))
C = float(input("Entrer la valeur de C (en farad) :    "))
nbpoints = int(input("Entrer le nombre de points de mesures : "))
port = str(input('Nom du port auquel est relié Arduino :   '))
FMesures = str(input('Entrer le nom du fichier de mesures : '))
FichierMesures = FMesures+'.txt'

#en option pour changer de répertoire par défaut qui est celui où est enregistré le programme
repertoire=str(input('Nom du repertoire de travail :  ')) 
os.chdir(repertoire) #change le répertoire de travail

# connexion et communication avec le port série avec réinitialisation
PortSerie = serial.Serial( port, baudrate =9600) # ouvre la connexion avec le port série et par défaut on utilise une communication à 9600 bauds
PortSerie .setDTR(False) 
time. sleep(0.1) # effectue une pause de 0,10 secondes
PortSerie .setDTR(True) 
PortSerie . flushInput () 
# On vide la mémoire tampon de la liaison série. Cela permet d’effacer d’éventuelles donnée de l’acquisition précendente restées en mémoire. 
# les mesures : génération d'une liste de mesures sous forme de tableau pour les variablles Uc, E et temps
Uc = [] 
temps = [] 
E = []
print("Veuillez patienter ....")
print("t   ","\t","uc   ","\t","e")

for i in range(nbpoints):
    val = PortSerie . readline (). split () 
    try: # try permet de gérer une erreur d’exécution
        t = float(val[0]) # valeurs de la première colonne 'démarre à 0'
        uc = float(val[1])
        e = float(val[2]) 
        temps.append(t) # on ajoute la valeur de t à la liste " temps"
        E.append(e) 
        Uc.append(uc)
        if  uc  <=0.01:
            break
        print(t,"\t",uc,"\t",e)
    except:
        pass # on ne fait rien  alors

# fermeture du port série 
PortSerie. close ()
 
t1,u0,u1=temps,E,Uc
# création d'un fichier texte avec les mesures
Mesures = open(FichierMesures,"w")
Mesures.write( "t\t"  + "Uc\t" + "E\n")
for j in range ( len(t1)):
    Mesures.write( str(t1[j])+ "\t" + str(u1[j]) +  "\t" + str(u0[j]) + "\n" )
Mesures.close()


# détermination de tau expérimental
def modele(t1,u):
   umax = max(u)
   for i in range(len(t1)-1):
       if u[i]>=umax*0.37 and u[i+1]<=umax*0.37:
            tauexp=(t1[i]+t1[i+1])/2
            return umax,tauexp

# calcul de tau theorique 
def calculTau(R,C):
    tau =R*C
    return tau

tau=calculTau(R,C)
Umax,tauexp=modele(t1,u1)
print("Umax  :\t",Umax," V")
print("tau experimental :\t",round(tauexp,3), "s") 
print("tau theorique : ",round(tau,3)," s")

# Modele 
def ucm(t):
    return Umax*np.exp(- t/tauexp)
    
Ucm=[]
for i in range(len(t1)):
    Ucm.append(ucm(t1[i]))

# Calcul de i
Ur=[]
for i in range(len(t1)):
    Ur.append(Umax-ucm(t1[i]))

# Tracé des 3 graphes
# Tracé temporel du signal 
fig1 = plt.subplot(221) 
plt.plot(t1,u0,"b*",label='e(t)')
plt.plot(t1,u1,"r+",label='uc(t)')
plt.xlabel("t (s)")
plt.ylabel("tension (V)")
plt.axis([0,t1[-1],0,5.5])
plt.title('uc(t) et e(t)')
plt.legend(loc='best')
# curseur pour lire une valeur
cursor = Cursor(fig1, useblit=True, color='blue', linewidth=1)
plt.grid()
# Tracé temporel du signal et modelisation
fig2 = plt.subplot(222)
plt.plot(t1,u1,"r+",label='uc(t)')
plt.plot(t1,Ucm,'b',label='ucm(t)')
plt.xlabel("t (s)")
plt.ylabel("tension (V)")
plt.axis([0,t1[-1],0,5.5])
plt.title('uc(t) et son modele)')
plt.legend(loc='best')
cursor = Cursor(fig2, useblit=True, color='green', linewidth=1)
plt.grid()
# Tracé temporel du signal et de i(t)
fig3 = plt.subplot(223)
plt.plot(t1,u1,"r+",label='uc(t)')
plt.plot(t1,Ur,'b',label='ur(t)')
plt.xlabel("t (s)")
plt.ylabel("tension (V)")
plt.legend(loc='best')
plt.title('uc(t) et ur(t)')
cursor = Cursor(fig3, useblit=True, color='black', linewidth=1)
plt.grid()
plt.show()
image = FMesures
plt.savefig(FMesures)
 # sauvegarde les graphes dans le fichier image au format png dans 
#le répertoire défini par l'opérateur avec un nom identique au nom du fichier de mesures